use std::fmt::Display;

use async_trait::async_trait;
use inquire::{ui::RenderConfig, Select};

use crate::flows::{add_game::AddGame, exit::Exit, list_games::ListGames};

use super::*;


#[derive(Clone)]
pub struct Games {
}

impl Games {
    pub fn new() -> Self {
        Self {}
    }
}

#[async_trait]
impl<'a> Flow<'a> for Games {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a> {
        let flows: Vec<Box<dyn Flow + Send>> = vec![
            Box::new(ListGames::new()),
            Box::new(AddGame::new()),
            Box::new(Exit::new())
        ];

        let choice = Select::new("What would you like to do?", flows)
            .with_help_message("Select the action you want to take or quit the program")
            .with_render_config(RenderConfig {
                option_index_prefix: inquire::ui::IndexPrefix::Simple,
                ..Default::default()
            })
            .prompt_skippable()?;
 
        self.continue_choice(state, &choice).await
    }
}

impl Display for Games {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Games")
    }
}