
use std::fmt::Display;

use inquire::{ui::RenderConfig, Select};

use crate::flows::{connect::Connect, exit::Exit, settings::Settings};

use super::*;

#[derive(Clone)]
pub struct Main {
}

impl Main {
    pub fn new() -> Self {
        Self {}
    }
}

impl Default for Main {
    fn default() -> Self {
        Self::new()
    }
}

#[async_trait]
impl<'a> Flow<'a> for Main {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a> {

        let config = Config::load()?;
        state.gamenight_configuration = config;

        let mut choices: Vec<Box<dyn Flow<'a> + Send>> = vec![];

        if let Some(last_instance) = &state.gamenight_configuration.last_instance {
            if let Some(instance) = state.gamenight_configuration.instances.iter().find(|instance| {instance.name.eq(last_instance)}) {
                choices.push(Box::new(Connect::to(instance.clone())));
            }
        }
        
        choices.push(Box::new(Connect::new()));
        choices.push(Box::new(Settings::new()));
        choices.push(Box::new(Exit::new()));

        let choice = Select::new("What would you like to do?", choices)
            .with_help_message("Select the action you want to take or quit the program")
            .with_render_config(RenderConfig {
                option_index_prefix: inquire::ui::IndexPrefix::Simple,
                ..Default::default()
            })
            .prompt_skippable()?;

        self.continue_choice(state, &choice).await
    }
}

impl Display for Main {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "main")
    }
}
