use diesel::{dsl::insert_into, ExpressionMethods, Insertable, PgConnection, QueryDsl, Queryable, RunQueryDsl};
use serde::{Serialize, Deserialize};
use uuid::Uuid;
use crate::schema::game;

use super::error::DatabaseError;

#[derive(Serialize, Deserialize, Debug, Insertable, Queryable)]
#[diesel(table_name = game)]
pub struct Game {
    pub id: Uuid,
    pub name: String,
}

pub fn games(conn: &mut PgConnection) -> Result<Vec::<Game>, DatabaseError> {
    Ok(game::table.load::<Game>(conn)?)
}

pub fn load_game(conn: &mut PgConnection, id: Uuid) -> Result<Game, DatabaseError> {
    Ok(game::table.find(id).get_result(conn)?)
}

pub fn insert_game(conn: &mut PgConnection, game: Game) -> Result<usize, DatabaseError> {
    Ok(insert_into(game::table).values(&game).execute(conn)?)
}

pub fn rename_game(conn: &mut PgConnection, id: Uuid, name: String) -> Result<usize, DatabaseError> {
    Ok(diesel::update(game::table.filter(game::id.eq(id)))
        .set(game::name.eq(&name))
        .execute(conn)?)   
}